# -*- coding: utf-8 -*-
"""This module contains exceptions.
"""


class ShowdownException(Exception):
    """This exception is raised when a non-managed message is received from the server.
    """

    pass


class UnexpectedEffectException(Exception):
    """This exception is raised when a non-managed effect is received from the server.
    """

    pass



# -*- coding: utf-8 -*-
"""This module contains constant values used in the repository.
"""

import json
import os

from typing import Any
from typing import Dict

from poke_env.utils import _compute_type_chart


_TYPE_CHART_PATH: str = os.path.join(
    os.path.dirname(os.path.realpath(__file__)), "data", "typeChart.json"
)
"Path to the json file containing type informations."

POKEDEX: Dict = {}

with open(
    os.path.join(os.path.dirname(os.path.realpath(__file__)), "data", "pokedex.json")
) as pokedex:
    POKEDEX = json.load(pokedex)

_missing_dex = {}
for key, value in POKEDEX.items():
    if "otherForms" in value:
        for other_form in value["otherForms"]:
            _missing_dex[other_form] = value

POKEDEX.update(_missing_dex)

_equivalent_forms = {"darmanitangalarzen": "darmanitanzengalar"}

POKEDEX.update({k: POKEDEX[v] for k, v in _equivalent_forms.items()})

MOVES: Dict = {}

with open(
    os.path.join(os.path.dirname(os.path.realpath(__file__)), "data", "moves.json")
) as moves:
    MOVES = json.load(moves)

TYPE_CHART: Dict[Any, Dict[Any, float]] = _compute_type_chart(_TYPE_CHART_PATH)
"""
A dictionnary representing the Pokemon type chart.

Each key is a string representing a type (corresponding to `Type` names), and each value
is a dictionnary whose keys are string representation of types, and whose values are
floats.

TYPE_CHART[type_1][type_2] corresponds to the damage multiplier of an attack of type_1
on a Pokemon of type_2. This dictionnary isncomputed using the `compute_type_chart`
function.
"""



# -*- coding: utf-8 -*-
"""This module contains utility functions and objects.
"""

import json

from typing import Dict


def _compute_type_chart(chart_path: str) -> Dict[str, Dict[str, float]]:
    """Returns the pokemon type_chart.

    Returns a dictionnary representing the Pokemon type chart, loaded from a json file
    in `data`. This dictionnary is computed in this file as `TYPE_CHART`.

    :return: The pokemon type chart
    :rtype: Dict[str, Dict[str, float]]
    """
    with open(chart_path) as chart:
        json_chart = json.load(chart)

    types = [str(entry["name"]).upper() for entry in json_chart]

    type_chart = {type_1: {type_2: 1.0 for type_2 in types} for type_1 in types}

    for entry in json_chart:
        type_ = entry["name"].upper()
        for immunity in entry["immunes"]:
            type_chart[type_][immunity.upper()] = 0.0
        for weakness in entry["weaknesses"]:
            type_chart[type_][weakness.upper()] = 0.5
        for strength in entry["strengths"]:
            type_chart[type_][strength.upper()] = 2.0

    return type_chart


def to_id_str(name: str) -> str:
    """Converts a full-name to its corresponding id string.
    :param name: The name to convert.
    :type name: str
    :return: The corresponding id string.
    :rtype: str
    """
    return "".join(char for char in name if char.isalnum()).lower()



# -*- coding: utf-8 -*-
"""This module contains utility functions and objects related to Player classes.
"""

from poke_env.player.player import Player
from poke_env.utils import to_id_str
from typing import Dict
from typing import List
from typing import Optional

import asyncio


async def cross_evaluate(
    players: List[Player], n_challenges: int
) -> Dict[str, Dict[str, Optional[float]]]:
    results = {p_1.username: {p_2.username: None for p_2 in players} for p_1 in players}
    for i, p_1 in enumerate(players):
        for j, p_2 in enumerate(players):
            if j <= i:
                continue
            await asyncio.gather(
                p_1.send_challenges(
                    opponent=to_id_str(p_2.username),
                    n_challenges=n_challenges,
                    to_wait=p_2.logged_in,
                ),
                p_2.accept_challenges(
                    opponent=to_id_str(p_1.username), n_challenges=n_challenges
                ),
            )
            results[p_1.username][p_2.username] = p_1.win_rate  # pyre-ignore
            results[p_2.username][p_1.username] = p_2.win_rate  # pyre-ignore

            p_1.reset_battles()
            p_2.reset_battles()
    return results  # pyre-ignore



# -*- coding: utf-8 -*-
"""This module contains objects related to player configuration.
"""
from collections import namedtuple

PlayerConfiguration = namedtuple("PlayerConfiguration", ["username", "password"])
"""Player configuration object. Represented with a tuple with two entries: username and
password."""
